/* filesel.c
 * Copyright (C) 2002-2003 Pascal Eberhard <pascal.ebo@netcourrier.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <stdio.h>      /* define FILENAME_MAX */

#include "common.h"

static void filesel_click_ok(filesel_t *filesel);
static void filesel_click_cancel(filesel_t *filesel);

// ----------------------------------------------------------------------------
filesel_t* filesel_new(apwalapp_t *apwal)
{
  filesel_t *filesel;
  g_assert(apwal != NULL);
  filesel = (filesel_t *) malloc(sizeof(filesel_t));
  g_assert(filesel != NULL);
  filesel->apwal = apwal;
  filesel->filesel = gtk_file_selection_new(NULL);
  gtk_widget_hide(filesel->filesel);
  g_signal_connect_swapped(G_OBJECT(GTK_FILE_SELECTION(
                           filesel->filesel)->ok_button), "clicked",
                           G_CALLBACK(filesel_click_ok), filesel);
  g_signal_connect_swapped(G_OBJECT(GTK_FILE_SELECTION(
                            filesel->filesel)->cancel_button), "clicked",
                            G_CALLBACK(filesel_click_cancel), filesel);
  return filesel;
}
// ----------------------------------------------------------------------------
void filesel_edit(filesel_t *filesel)
{
  gchar *oldfile;
  g_assert(filesel != NULL);
  g_assert(filesel->apwal != NULL);
  g_assert(filesel->apwal->selected_app != NULL);
  g_assert(filesel->apwal->selected_app_field != -1);
  
  switch(filesel->apwal->selected_app_field)
  {
  case APP_FIELD_EXEC: break;
  case APP_FIELD_PATH: break;
  default: ERR("app_field:%d", filesel->apwal->selected_app_field);
  }
  oldfile = app_field_get_value(filesel->apwal->selected_app,
                                filesel->apwal->selected_app_field);
  if (oldfile == NULL)
    ERR("oldfile NULL, app:%p, field:%d", filesel->apwal->selected_app,
        filesel->apwal->selected_app_field);

  gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel->filesel),
                                  oldfile);
  gtk_grab_add(filesel->filesel); //modal
  gtk_widget_show(filesel->filesel);
}
// ----------------------------------------------------------------------------
static void filesel_click_ok(filesel_t *filesel)
{
  const gchar *file;
  g_assert(filesel != NULL);
  g_assert(filesel->apwal != NULL);
  g_assert(filesel->apwal->selected_app != NULL);
  g_assert(filesel->apwal->selected_app_field != -1);
  
  file = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel->filesel));
  gtk_grab_remove(filesel->filesel); /*modal*/
  gtk_widget_hide(filesel->filesel);
  
  switch(filesel->apwal->selected_app_field)
  {
  case APP_FIELD_EXEC: break;
  case APP_FIELD_PATH: break;
  default: ERR("app_field:%d", filesel->apwal->selected_app_field);
  }
  app_field_set_value(filesel->apwal->selected_app,
                      filesel->apwal->selected_app_field, (char *)file);
  apwalapp_selected_app_modified(filesel->apwal);
}
// ----------------------------------------------------------------------------
void filesel_click_cancel(filesel_t *filesel)
{
  g_assert(filesel != NULL);
  g_assert(filesel->apwal != NULL);
  g_assert(filesel->apwal->selected_app != NULL);
  g_assert(filesel->apwal->selected_app_field != -1);
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel->filesel), "");
  gtk_grab_remove(filesel->filesel); /*modal*/
  gtk_widget_hide(filesel->filesel);
}
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
