/* preference.c
 * Copyright (C) 2002-2003 Pascal Eberhard <pascal.ebo@netcourrier.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "common.h"

// ----------------------------------------------------------------------------
typedef struct preferences_t
{
  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *bbox, *button_cancel, *button_ok;
} preferences_t;

// ----------------------------------------------------------------------------
preferences_t * preferences_new(void)
{
  preferences_t *pref;

  pref = (preferences_t *)malloc(sizeof(preferences_t));
  g_assert(pref != NULL);

  pref->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_resizable(GTK_WINDOW(pref->window), FALSE);
  g_signal_connect_swapped(G_OBJECT(pref->window), "delete_event",
                           G_CALLBACK(preferences_hide), pref);

  pref->vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(pref->vbox), 10);
  gtk_container_add(GTK_CONTAINER(pref->window), pref->vbox);
  gtk_widget_show(pref->vbox);

  // button ok and cancel
  pref->bbox = gtk_hbutton_box_new();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(pref->bbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(pref->bbox), 10);

  gtk_box_pack_start(GTK_BOX(pref->vbox), pref->bbox, FALSE, FALSE, 0);
  gtk_widget_show(pref->bbox);

  pref->button_cancel = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
  GTK_WIDGET_SET_FLAGS(pref->button_cancel, GTK_CAN_DEFAULT);
  g_signal_connect_swapped(G_OBJECT(pref->button_cancel), "clicked",
                           G_CALLBACK(preferences_hide), pref);
  gtk_box_pack_start(GTK_BOX(pref->bbox), pref->button_cancel, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS(pref->button_cancel, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(pref->button_cancel);
  gtk_widget_show(pref->button_cancel);

  pref->button_ok = gtk_button_new_from_stock(GTK_STOCK_OK);
  GTK_WIDGET_SET_FLAGS(pref->button_ok, GTK_CAN_DEFAULT);
  g_signal_connect_swapped(G_OBJECT(pref->button_ok), "clicked",
                           G_CALLBACK(preferences_hide), pref);
  gtk_box_pack_start(GTK_BOX(pref->bbox), pref->button_ok, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS(pref->button_ok, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(pref->button_ok);
  gtk_widget_show(pref->button_ok);

  return pref;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
